<?php

/**
 * Class for interaction with Site Search 360 API.
 */
class SiteSearch360 {

  /**
   * The base url to use.
   * @var string
   */
  private $baseUrl = NULL;

  /**
   * The API key to use.
   * @var string
   */
  private $key = NULL;

  /**
   * The search query.
   * @var string
   */
  private $query = NULL;

  /**
   * Extra parameters to be passed along.
   * @var array
   */
  private $extraParams = NULL;

  /**
   * Page size, number of results per page.
   * @var int
   */
  private $pageSize = 20;

  /**
   * Pager size, number of pages to show in the pager.
   * @var int
   */
  private $pagerSize = 19;

  /**
   * Current page in the results.
   * @var int
   */
  private $currentPage = NULL;

  /**
   * Total results for a search.
   * @var int
   */
  private $totalResults = 0;

  /**
   * Create a new instance of SiteSearch360 class.
   *
   * @param string
   */
  public function __construct($baseUrl, $query, $key = NULL, $pageSize = 20, $extraParams = NULL) {
    $this->baseUrl = $baseUrl;
    $this->query = $query;
    $this->key = $key;
    $this->pageSize = $pageSize;
    $this->extraParams = $extraParams;
  }

  /**
   * Get the base url
   *
   * @return string The base url.
   */
  public function getBaseUrl() {
    return $this->baseUrl;
  }

  /**
   * Set the base url.
   *
   * @param string $key The API key.
   * @return void
   */
  public function setBaseUrl($baseUrl) {
    $this->baseUrl = $baseUrl;
  }

  /**
   * Set the API key.
   *
   * @param string $key The API key.
   * @return void
   */
  public function setKey($key) {
    $this->key = $key;
  }

  /**
   * Get API key
   *
   * @return string The API key.
   */
  public function getKey() {
    return $this->key;
  }

  /**
   * Set the page size.
   *
   * @param int $pageSize The new page size.
   * @return void
   */
  public function setPageSize($pageSize) {
    $this->pageSize = $pageSize;
  }

  /**
   * Set the pager size.
   *
   * @param int $pagerSize The new pager size.
   * @return void
   */
  public function setPagerSize($pagerSize) {
    $this->pagerSize = $pagerSize;
  }

  /**
   * Get current page size.
   *
   * @return int The current page size.
   */
  public function getPageSize() {
    return $this->pageSize;
  }

  /**
   * Get current pager size.
   *
   * @return int The current pager size.
   */
  public function getPagerSize() {
    return $this->pagerSize;
  }

  /**
   * Get current page in the results.
   *
   * @return int The current page.
   */
  public function getCurrentPage() {
    return $this->currentPage;
  }

  /**
   * Get total results for current search.
   *
   * @return int Total number of results.
   */
  public function getTotalResults() {
    return $this->totalResults;
  }

  /**
   * Get total number of pages for search.
   *
   * @return int Total number of pages.
   */
  public function getTotalPages() {
    return ($this->totalResults / $this->pageSize + 1);
  }

  /**
   * Get query current search.
   *
   * @return string The current query.
   */
  public function getQuery() {
    return $this->query;
  }

  /**
   * Get query extra parameters array.
   *
   * @return array The extra parameters.
   */
  public function getExtraParameters() {
    return $this->extraParams;
  }

  /**
   * Get search results.
   *
   * @param string $query The search query.
   * @param int $page The page to get results from.
   * @return array The search results.
   * @throws Exception
   */
  public function getSearchResults($page = 1) {

    // set the current page
    $this->currentPage = $page;

    // calculate start postition based on page
    $startPos = ($this->currentPage - 1) * $this->pageSize;

    $params = [
      'query' => [
        'query' => $this->query,
        'limit' => $this->pageSize,
        'offset' => $startPos
      ]
    ];

    // prepare query parameters for URL assembly
    if (count($this->extraParams) > 0) {
      $params['query']  = array_merge($params['query'], $this->extraParams);
    }
    else {
      $extraParamsQuery = NULL;
    }

    // fetch results from the API
    $response = drupal_http_request(url($this->baseUrl, $params));

    if ($response->code != 200) {
      throw new Exception('Could not load search results from the API.');
    }

    $results = json_decode($response->data, TRUE);

    // no results, return NULL
    if (empty($results['suggests'])) {
      return NULL;
    }

    // save total results
    $this->totalResults = $results['totalResults'];

    return $results['suggests'];
  }

  /**
   * Get search suggests.
   *
   * @return array The search suggests.
   * @throws Exception
   */
  public function getSuggests() {

    $params = [
      'query' => [
        'query' => $this->query,
      ]
    ];

    // prepare query parameters for URL assembly
    if (count($this->extraParams) > 0) {
      $params['query']  = array_merge($params['query'], $this->extraParams);
    }
    else {
      $extraParamsQuery = NULL;
    }

    // fetch results from the API
    $response = drupal_http_request(url($this->baseUrl, $params));

    if ($response->code != 200) {
      throw new Exception('Could not load search results from the API.');
    }

    $results = json_decode($response->data, TRUE);

    // no suggests, return NULL
    if (empty($results['suggests'])) {
      return NULL;
    }

    return $results['suggests'];
  }

  /**
   * Get a pager for the search results.
   *
   * @return string The pager HTML code.
   */
  public function getPager($link = '?page=') {

    // get total pages
    $totalPages = $this->getTotalPages();
    $currentPage = $this->getCurrentPage();
    $pagerSize = $this->getPagerSize();
    $halfPagerSize = $pagerSize / 2;

    if ($totalPages < 2) {
      return;
    }

    $html = "<div class=\"item-list ss360-pager\"><ul class=\"pager\">\n";

    // Limit the number of pages to 50
    $totalPages = $totalPages > 50 ? 50 : $totalPages;

    if ($currentPage > 1) {
      $link_page = $link . ($currentPage - 1);
      $html .= "\t<li class=\"pager-previous\"><a href=\"{$link_page}\">" . t('‹ previous') . "</a></li>\n";
    }

    // Defines the start and finish of the pager
    $pagerStart = $currentPage - (round($pagerSize / 2) - 1);
    $pagerFinish = $currentPage + (round($pagerSize / 2) - 1);

    // Start values, if $pagerStart is negative
    if ($pagerStart < 1) {
      $pagerStart = 1;
      $pagerFinish = $pagerSize > $totalPages ? $totalPages : $pagerSize;
    }

    // Limit $pagerFinish to the same as $totalPages
    $pagerFinish = $pagerFinish > $totalPages ? $totalPages : $pagerFinish;

    if ($pagerStart > 1) {
      $html .= "\t<li class=\"pager-ellipsis\">...</li>\n";
    }

    for ($i = $pagerStart; $i <= $pagerFinish; $i++) {
      if ($i != $currentPage) {
        $link_page = $link . $i;
        $html .= "\t<li class=\"pager-item\"><a href=\"{$link_page}\">{$i}</a></li>\n";
      }
      else {
        $html .= "\t<li class=\"pager-current\">{$i}</li>\n";
      }
    }

    if ($pagerFinish < $totalPages) {
      $html .= "\t<li class=\"pager-ellipsis\">...</li>\n";
    }

    if ($currentPage < intval($totalPages)) {
      $link_page = $link . ($currentPage + 1);
      $html .= "\t<li class=\"pager-next\"><a href=\"{$link_page}\">" . t('next ›') . "</a></li>\n";
    }

    // close ol
    $html .= "</ul></div>\n";

    // return pager
    return $html;
  }

  /**
   * Get a head to the search results.
   *
   * @return string The search head HTML code.
   */
  public function getSearchHead() {
    // get total pages
    $totalResults = $this->getTotalResults();
    $pageSize = $this->getPageSize();
    $currentPage = $this->getCurrentPage();
    $show_start = ($pageSize * $currentPage) - $pageSize + 1;
    $show_end = ($pageSize * $currentPage);
    $query = $this->getQuery();

    if ($show_end > $totalResults) {
      $show_end = $totalResults;
    }

    $html = '<div class="searchhead">';

    // TODO: move variable fetching outside of the class
    if (variable_get('ss360_number_of_results', TRUE) == 1 && $totalResults !== 0) {
      $html .= t('Shows @show_start to @show_end of approximately @totalResults hits for "@query"',
        [
          '@show_start' => $show_start,
          '@show_end' => $show_end,
          '@totalResults' => $totalResults,
          '@query' => check_plain($query)
        ]);
    }

    $html .= '</div>';

    // return search head
    return $html;
  }
}
